part of CoreView;

/**
 * Does support async rendering;
 * Does support async content pulling;
 * Does support async data pulling;
 */
class MustacheView<T> extends DataView {
  Function  _dataPuller;
  String    _content;
  Function  _contentPuller;
  String    _scope;
  
  MustacheView(Function dataPuller, {String content: ""}) {
    this._dataPuller = dataPuller;
    this._content = content;
  } 
  
  String      getContent() => this._content;
  Renderable  setContent(String content) {
    this._content = content;
    return this;
  }
         
  void        setPuller(Function puller) { this._dataPuller = puller; }
  Function    getPuller() => this._dataPuller;
  String get  pull => this._contentPuller();
  void        depullerize() { this._contentPuller = null; }
  
  void  setScope(String selector) { this._scope = selector; }
  String  getScope() => this._scope;
  
  Future<String>  render() async {
    // Prioritize a puller over the static-content
    String content;
    if(this._contentPuller is Function) content = await this._contentPuller();
    else content = this._content;
     
    var template = mustache.parse(content);
    Map data = await this._dataPuller();
    return template.renderString(data);
  }
}